@@ -1,10 +1,13 @@
 """SciUnit tests live in this module."""
 
-from copy import deepcopy
 import inspect
 import traceback
+from uuid import uuid4
+from copy import deepcopy
 from typing import Any, List, Optional, Tuple, Union
 
+import quantities as pq
+
 from .base import SciUnit, config
 from .capabilities import ProducesNumber
 from .errors import (
@@ -16,7 +19,7 @@ from .errors import (
 )
 from .models import Model
 from .scores import BooleanScore, ErrorScore, NAScore, NoneScore, Score, TBDScore
-from .utils import dict_combine
+from .utils import dict_combine, use_backend_cache
 from .validators import ObservationValidator, ParametersValidator
 
 
@@ -41,6 +44,8 @@ class Test(SciUnit):
         if self.description is None:
             self.description = self.__class__.__doc__
 
+        self.id = uuid4().hex
+
         # Use a combination of default_params and params, choosing the latter
         # if there is a conflict.
         self.params = dict_combine(self.default_params, params)
@@ -50,7 +55,7 @@ class Test(SciUnit):
         self.compute_params()
 
         self.observation = observation
-        
+
         # Fall back to the score class's observation_schema if the test class doesn't have one
         if self.observation_schema is None:
             self.observation_schema = deepcopy(self.score_type.observation_schema)
@@ -88,16 +93,18 @@ class Test(SciUnit):
     Can also be a list of schemas, one of which the observation must match.
     If it is a list, each schema in the list can optionally be named by putting
     (name, schema) tuples in that list."""
-    
+
     observation_validator = ObservationValidator
 
     params_schema = None
     """A schema that the params must adhere to (validated by cerberus).
     Can also be a list of schemas, one of which the params must match."""
-    
+
     params_validator = ParametersValidator
-    
-    state_hide = ['last_model']
+
+    units = pq.dimensionless
+
+    state_hide = ["last_model"]
 
     def compute_params(self) -> None:
         """Compute new params from existing `self.params`.
@@ -106,7 +113,6 @@ class Test(SciUnit):
         Example: `self.params['c'] = self.params['a'] + self.params['b']`
 
         """
-        pass
 
     def validate_observation(self, observation: dict) -> dict:
         """Validate the observation provided to the constructor.
@@ -253,8 +259,8 @@ class Test(SciUnit):
         Args:
             model (Model): A sciunit model instance.
         """
-        pass
 
+    @use_backend_cache
     def generate_prediction(self, model: Model) -> None:
         """Generate a prediction from a model using the required capabilities.
 
@@ -278,7 +284,6 @@ class Test(SciUnit):
         Args:
             prediction (float): The predicted value.
         """
-        pass
 
     def compute_score(self, observation: dict, prediction: dict) -> Score:
         """Generates a score given the observations provided in the constructor
@@ -357,7 +362,6 @@ class Test(SciUnit):
             observation (Union[list, dict]): The observation data.
             prediction (Union[list, dict]): The prediction data.
         """
-        pass
 
     def check_score_type(self, score: Score) -> None:
         """Check that the score is the correct type for this test.
@@ -375,7 +379,9 @@ class Test(SciUnit):
             ) % (self.name, self.score_type.__name__, score.__class__.__name__)
             raise InvalidScoreError(msg)
 
-    def _judge(self, model: Model, skip_incapable: bool = True, cached_prediction=False) -> Score:
+    def _judge(
+        self, model: Model, skip_incapable: bool = True, cached_prediction=False
+    ) -> Score:
         """Generate a score for the model (internal API use only).
 
         Args:
@@ -430,6 +436,22 @@ class Test(SciUnit):
                           deep_error=deep_error, cached_prediction=True)
 
 
+    def feature_judge(
+        self,
+        model: Model,
+        skip_incapable: bool = False,
+        stop_on_error: bool = True,
+        deep_error: bool = False,
+    ) -> Score:
+        """For backwards compatibility"""
+        return self.judge(
+            model,
+            skip_incapable=skip_incapable,
+            stop_on_error=stop_on_error,
+            deep_error=deep_error,
+            cached_prediction=True,
+        )
+
     def judge(
         self,
         model: Model,
@@ -488,12 +510,18 @@ class Test(SciUnit):
             )
 
         if deep_error:
-            score = self._judge(model, skip_incapable=skip_incapable,
-                                cached_prediction=cached_prediction)
+            score = self._judge(
+                model,
+                skip_incapable=skip_incapable,
+                cached_prediction=cached_prediction,
+            )
         else:
             try:
-                score = self._judge(model, skip_incapable=skip_incapable,
-                                    cached_prediction=cached_prediction)
+                score = self._judge(
+                    model,
+                    skip_incapable=skip_incapable,
+                    cached_prediction=cached_prediction,
+                )
             except CapabilityError as e:
                 score = NAScore(str(e))
                 score.model = model
@@ -575,6 +603,45 @@ class Test(SciUnit):
                 result = "\n".join(s)
         return result
 
+    def get_backend_cache(self, model: Model, key: Optional[str]=None) -> Any:
+        """Get the cached results from the model's backend with the given key
+        (defaults to the id of the test instance).
+
+        Returns:
+            Any: The cache for key 'key' or None if not found.
+        """
+        if model is None:
+            return None
+        if key is None:
+            if hasattr(self, 'id'):
+                key = self.id
+            else:
+                return None
+
+        if hasattr(model, 'backend') and not model.backend is None:
+            return model._backend.get_cache(key=key)
+        return None
+
+    def set_backend_cache(self, model: Model, function_output: Any,
+                  key: Optional[str]=None) -> bool:
+        """Set the cache of the model's backend with the given key (defaults to
+        the id of the test instance)to calculated function output.
+
+        Returns:
+            bool: True if cache was successfully set, else False
+        """
+        if model is None:
+            return False
+        if key is None:
+            if hasattr(self, 'id'):
+                key = self.id
+            else:
+                return False
+
+        if hasattr(model, 'backend') and model.backend is not None:
+            return model._backend.set_cache(function_output, key=key)
+        return False
+
     @property
     def state(self) -> dict:
         """Get the frozen (pickled) model state.
@@ -628,7 +695,6 @@ class TestM2M(Test):
         Args:
             observation (dict): The observation to be validated.
         """
-        pass
 
     def compute_score(self, prediction1: dict, prediction2: dict) -> Score:
         """Generate a score given the observations provided in the constructor
@@ -709,7 +775,6 @@ class TestM2M(Test):
             model1 (Model): The first model.
             model2 (Model): The second model.
         """
-        pass
 
     def _judge(
         self, prediction1, prediction2, model1: Model, model2: Model = None
