@@ -5,14 +5,16 @@ tests located in other modules.
 """
 
 from .backend_tests import *
+from .base_tests import *
 from .command_line_tests import *
 from .config_tests import *
 from .converter_tests import *
 from .doc_tests import *
 from .error_tests import *
 from .import_tests import *
-from .observation_tests import *
 from .model_tests import *
+from .observation_tests import *
 from .score_tests import *
 from .test_tests import *
 from .utils_tests import *
+from .validator_tests import *
