@@ -7,10 +7,6 @@ import functools
 import hashlib
 import importlib
 import inspect
-import json
-import jsonpickle
-import logging
-import math
 import os
 import pkgutil
 import re
@@ -19,28 +15,36 @@ import traceback
 import unittest.mock
 import warnings
 from datetime import datetime
-
 from io import StringIO, TextIOWrapper
 from pathlib import Path
 from types import ModuleType
 from typing import Any, Callable, List, TextIO, Tuple, Type, Union
+from urllib.request import urlopen
 
-import bs4
+import jsonpickle
 import nbconvert
 import nbformat
-from IPython.display import HTML, display        
+from IPython.display import HTML, display
 from nbconvert.preprocessors import ExecutePreprocessor
 from nbconvert.preprocessors.execute import CellExecutionError
 from quantities.dimensionality import Dimensionality
 from quantities.quantity import Quantity
 
 import sciunit
-from sciunit.errors import Error
 
-from .base import PLATFORM, PYTHON_MAJOR_VERSION, __version__, SciUnit, tkinter, config, ipy
+from .base import (  # noqa
+    PLATFORM,
+    PYTHON_MAJOR_VERSION,
+    SciUnit,
+    __version__,
+    config,
+    ipy,
+    tkinter,
+)
 
 mock = False  # mock is probably obviated by the unittest -b flag.
 
+
 def warn_with_traceback(
     message: str,
     category: Type[Warning],
@@ -280,6 +284,7 @@ class NotebookTools(object):
         """
         self.convert_notebook(name)
         code = self.read_code(name)
+        code = "from IPython import InteractiveShell as get_ipython\n" + code
         exec(code, globals())
 
     def gen_file_path(self, name: str) -> Path:
@@ -463,6 +468,22 @@ class MockDevice(TextIOWrapper):
         if s.startswith("[") and s.endswith("]"):
             super(MockDevice, self).write(s)
 
+class TmpTestFolder():
+    '''A class for creating and deleting a folder in "./unit_test/".
+    '''
+
+    path = Path(__file__).parent / "unit_test" / "delete_after_tests"
+
+    def __init__(self, location: Union[str, Path, None] = None) -> None:
+        if location: self.path = Path(location)
+
+    def create(self) -> None:
+        Path(self.path).mkdir(parents=True, exist_ok=True)
+
+    def delete(self) -> None:
+        import shutil
+        if self.path.exists() and self.path.is_dir():
+            shutil.rmtree(self.path)
 
 def import_all_modules(
     package, skip: list = None, verbose: bool = False, prefix: str = "", depth: int = 0
@@ -1005,24 +1026,101 @@ def memoize(fn=None):
 
     return decorated
 
-
 class_intern = intern.intern
 
 method_memoize = memoize
 
+def use_backend_cache(original_function=None, cache_key_param=None):
+    """
+    Decorator for test functions (in particular `generate_prediction`) to cache
+    the function output on the first execution and return the output from the
+    cache without recomputing on any subsequent execution.
+    The function needs to take a model as an argument, and the caching relies on
+    the model's backend. If it doesn't have a backend the caching step is
+    skipped.
+    Per default, a test instance specific hash is used to link the model to the
+    test's function output. However, optionally, a custom hash key name can be
+    passed to the decorator to use the hash stored in
+    `self.params[<hash key name>]` instead (e.g. for using a shared cache for
+    redundant calculations on the same model across tests).
+    """
+
+    def _decorate(function):
+
+        @functools.wraps(function)
+        def wrapper(self, *args, **kwargs):
+            sig = inspect.signature(function)
+            if 'model' in kwargs:
+                model = kwargs['model']
+            elif 'model' in sig.parameters.keys():
+                model = args[list(sig.parameters.keys()).index('model')-1]
+            else:
+                model = None
+                warnings.warn("The decorator `use_backend_cache` can only "
+                              "be used for test class functions that get "
+                              "'model' as an argument! Caching is skipped.")
+
+            cache_key = None
+            if cache_key_param:
+                if cache_key_param in self.params:
+                    cache_key = self.params[cache_key_param]
+                else:
+                    model = None
+                    warnings.warn("The value for the decorator arguement "
+                                  "cache_key_param value can not be found in "
+                                  "self.params! Caching is skipped.")
+
+            function_output = self.get_backend_cache(model=model,
+                                                     key=cache_key)
+
+            if function_output is None:
+                function_output = function(self, *args, **kwargs)
+                self.set_backend_cache(model=model,
+                                       function_output=function_output,
+                                       key=cache_key)
+
+            return function_output
+
+        return wrapper
+
+    if original_function:
+        return _decorate(original_function)
+    else:
+        return _decorate
 
 def style():
     """Style a notebook with the current sciunit CSS file"""
-    path = Path(__file__).parent / 'style.css'
-    with open(path, 'rb') as f:
-        css_style = f.read().decode('utf-8')
-        display(HTML("""
-            <style>  
-            %s
-            </style>
-            """ % css_style))
+
+    # Try a custom one in the user's home directory
+    path = Path.home() / ".sciunit" / "style.css"
+
+    # Try the one in the currently cloned sciunit repo
+    if not path.is_file():
+        path = Path(__file__).parent / "style.css"
+
+    if path.is_file():  # Load from disk
+        with open(path, "rb") as f:
+            css_style = f.read().decode("utf-8")
+    else:  # Load from the sciunit github repo
+        url = (
+            "https://raw.githubusercontent.com/scidash/sciunit/master/sciunit/style.css"
+        )
+        response = urlopen(url)
+        css_style = response.read().decode("utf-8")
+
+    # Apply the style in the notebook
+    display(
+        HTML(
+            """
+                 <style>
+                 %s
+                 </style>
+                 """
+            % css_style
+        )
+    )
 
 
 def dict_hash(d):
     s = jsonpickle.encode(d)
-    return hashlib.sha224(s.encode('latin1')).hexdigest()
\ No newline at end of file
+    return hashlib.sha224(s.encode("latin1")).hexdigest()
