@@ -81,10 +81,13 @@ class ObservationValidator(Validator):
             else:
                 required_units = self.test.units
             if not isinstance(value, pq.quantity.Quantity):
-                self._error(key, "Must be a python quantity")
+                #self._error(key, "Must be a python quantity")
+                value *= pq.dimensionless
             provided_units = value.simplified.units
             if not isinstance(required_units, pq.Dimensionless):
                 required_units = required_units.simplified.units
+            else:
+                required_units = required_units.units
             if not required_units == provided_units:
                 self._error(key, "Must have units of '%s'" % self.test.units.name)
 
